#ifndef _ALibObject_hpp_
#define _ALibObject_hpp_

// Copyright (c) 1997-1999 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@askesis.nl

// Author : Joost Kraaijeveld
// Created: 26 Nov 1997 08:26:30
// Purpose: This class is a base class for every object in the library. Any library global stuff
//          can be  put here. It also defines the asString/asDebugInfo/asChar/asDebugChar
//          and the getObjectType interface

// DLL Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers
#include <iostream>  // Isn't iostream depricated????
#include <typeinfo>

// Operating system toolkit headers

// Stl headers
#include <string>

// Other class headers

// Forward declarations

class /*ALIBIMPEXP*/ ALibObject
{
   public:
      // AVersion struct for library versioning
      struct AVersion
      {
         AVersion(unsigned short aMajor,unsigned short aMinor) : major(aMajor),minor(aMinor){}
         unsigned short major;
         unsigned short minor;
      };

      // Constructors
      ALibObject();
      ALibObject( const ALibObject& aALibObject);

      // Destructor
      virtual ~ALibObject();

      // Operators
      ALibObject& operator=( const ALibObject& aALibObject);
      bool operator==(const ALibObject& aALibObject) const;
      bool operator<(const ALibObject& aALibObject) const;

      // Accessors
      static AVersion                           getLibraryVersion();

      // Mutators

      // Other operations
      virtual string                            asString() const;
      virtual string                            asDebugInfo() const;
      virtual const char*                       asChar() const;
      virtual const char*                       asDebugChar() const;

      // Attributes

   protected:
      // Attributes

      // Operations

   private:
      // Friends
      friend ostream& operator<<(ostream& os,const ALibObject& aALibObject);

      // Operations

      // Attributes
}; // End of class declaration of ALibObject

inline ostream& operator<<(ostream& os,const ALibObject& aALibObject)
{
   os << aALibObject.asString();
   return os;
}

#endif //_ALibObject_hpp_
