#ifndef _ADate_hpp_
#define _ADate_hpp_

// Copyright (c) 1997-1999 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to JKraaijeveld@Askesis.nl

// Author : Joost Kraaijeveld
// Created: 26-09-97 14:12:24
// Purpose:

// Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <iostream.h>
#include <time.h>

// Operating system toolkit headers

// STL headers

// Other class headers
#include "ALibObject.hpp"

// Forward declarations
enum wxdate_format_type {wxMDY, wxDAY, wxMONTH, wxFULL, wxEUROPEAN};

typedef enum { yy , yyyy } YearFormat;

#define wxNO_CENTURY  0x02
#define wxDATE_ABBR   0x04

class ALIBIMPEXP ADate : virtual public ALibObject
{
   public:
      // Constructors
      ADate();
      ADate(const long aJulianDate);
      ADate(const int aDay,
            const int aMonth,
            const int aYear);
      ADate(const char* aFormatedDateString);  // yyyy-mm-dd
      ADate(const ADate &aADate);

      // Destructor

      // Operators
      operator string(void);
      ADate& operator=(const ADate& aADate);

      ADate operator+(const long i);
      ADate operator+(const int  i);

      ADate operator-(const long i);
      ADate operator-(const int  i);

      long operator-(const ADate &dt);

      ADate& operator+=(const long i);
      ADate& operator-=(const long i);

      ADate& operator++();     // Prefix increment
      ADate& operator++(int);  // Postfix increment
      ADate& operator--();     // Prefix decrement
      ADate& operator--(int);  // Postfix decrement

      bool operator<(  const ADate& aADate) const;
      bool operator<=( const ADate& aADate) const;
      bool operator>(  const ADate& aADate) const;
      bool operator>=( const ADate& aADate) const;
      bool operator==( const ADate& aADate) const;
      bool operator!=( const ADate& aADate) const;

      // Accessors
      long                                      getJulianDate() const;
      int                                       getYear() const;
      int                                       getMonth() const;                      // Month number (1..12)
      int                                       getDay() const;                        // Day in month number (1..31)

      // Mutators
      void                                      setNullDate(bool aNullDate = true){nullDate = aNullDate;}

      // Other operations
      // IDate function
//      IString                                   asString() const;
//      IString                                   asString(YearFormat yearFmt) const;
//      IString                                   asString(const char* fmt) const;
      // ADate functions
      string                                    formatDate( const int type=-1) const;
      void                                      setFormat(  const int format);
      int                                       setOption(  const int option,
                                                            const bool enable = true);

      int                                       getDayOfYear() const;                  // returns relative date since Jan. 1
      bool                                      isLeapYear() const;                    // returns TRUE if leap year, FALSE if not

      ADate&                                    set();                                 // Sets to current system date
      ADate&                                    set(  long lJulian);
      ADate&                                    set(  int aDay,
                                                      int aMonth,
                                                      int aYear);

      ADate&                                    addWeeks( int nCount = 1);             //
      ADate&                                    addMonths(int nCount = 1);             // May also pass neg# to decrement
      ADate&                                    addYears(int nCount = 1);              //

      int                                       getDaysInMonth();                      // Number of days in month (1..31)
      int                                       getFirstDayOfMonth() const;            // First Day Of Month  (1..7)

      string                                    getDayOfWeekName();                    // Character Day Of Week ('Sunday'..'Saturday')
      int                                       getDayOfWeek() const;                  // (1..7)

      int                                       getWeekOfMonth();                      // Numeric Week Of Month  (1..6)
      int                                       getWeekOfYear();                       // Numeric Week Of Year   (1..52)

      string                                    getMonthName();                        // Character Month name
      ADate                                     getMonthStart();                       // First Date Of Month
      ADate                                     getMonthEnd();                         // Last Date Of Month

      ADate                                     getYearStart();                        // First Date Of Year
      ADate                                     getYearEnd();                          // Last Date Of Year

      bool                                      isBetween(const ADate& first, const ADate& second) const;

      ADate                                     previous(const int dayOfWeek) const;
      bool                                      isValid() const;

      // Attributes

   protected:
      // Attributes
      unsigned long                             julian;           // days since 1/1/4713 B.C.
      int                                       month;
      int                                       day;
      int                                       year;
      int                                       day_of_week;      // 1 = Sunday, ... 7 = Saturday

      // Operations

   private:
      // Friends
      friend ostream& operator<<(ostream& os,const ADate& aADate);

      // Operations
      void                                      julian_to_mdy ();         // convert julian day to mdy
      void                                      julian_to_wday ();        // convert julian day to day_of_week
      void                                      mdy_to_julian ();         // convert mdy to julian day

      // Attributes
      int                                       displayFormat;
      unsigned char                             displayOptions;
      bool                                      nullDate;
};

#endif
