#include "ATime.hpp"

// Copyright (c) 1997-1999 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to JKraaijeveld@Askesis.nl

// Author : Joost Kraaijeveld
// Created: 11 Oct 1997 14:21:38

// public
// Constructors
ATime::ATime()
   :  nullTime(false)
{
   ticks = ATime::now().ticks;
}

ATime::ATime(  long aSeconds)
   :  nullTime(false)
{
   ticks = aSeconds;
   initialize(ticks);
}

ATime::ATime(  const char* aFormatedTimeString)
   :  nullTime(false)
{
   if(strlen(aFormatedTimeString)==0)
   {
      ticks = 0;
      nullTime = true;
   }else
   {
      //  01234567
      // "11.11.11"
      //  hh.mm.ss
      unsigned long hours     = atoi(aFormatedTimeString+0);
      unsigned long minutes   = atoi(aFormatedTimeString+3);
      unsigned long seconds   = atoi(aFormatedTimeString+6);
      ticks = ATime(hours,minutes,seconds).ticks;
   }
}

ATime::ATime(  unsigned long aHour,
               unsigned long aMinutes,
               unsigned long aSeconds)
   :  nullTime(false)
{
   ticks = aHour;
   ticks *= 60;
   ticks += aMinutes;
   ticks *= 60;
   ticks += aSeconds;

   // should we call initialize(ticks) as hour may be >24 or assert aHour < 24, aMinutes < 60 and aSeconds < 60???
   // initialize(ticks);
}

ATime::ATime(const ATime& aATime)
   :  ticks(aATime.ticks),
      nullTime(aATime.nullTime)
{
}

// Destructor

// Operators
ATime& ATime::operator=(const ATime& aATime)
{
   ticks    = aATime.ticks;
   nullTime = aATime.nullTime;
   return *this;
}

bool ATime::operator==(const ATime& aATime) const
{
   return (ticks == aATime.ticks) && (nullTime == aATime.nullTime);
}

ATime& ATime::operator+=(long aSeconds)
{
   ticks += aSeconds;
   initialize(ticks);
   return *this;
}

ATime& ATime::operator+=(const ATime& aATime)
{
   ticks += aATime.ticks;
   initialize(ticks);
   return *this;
}

ATime& ATime::operator-=(long aSeconds)
{
   ticks -= aSeconds;
   initialize(ticks);
   return *this;
}

ATime& ATime::operator-=(const ATime& aATime)
{
   ticks -= aATime.ticks;
   initialize(ticks);
   return *this;
}

// Accessors

// Mutators

// Other operations
// ITime functions
//IString ATime::asString() const
//{
//   return IString(formatTime());
//}

//IString ATime::asString( const char* /*fmt*/) const
//{
//   return IString(formatTime());
//}

/* static */ ATime ATime::now()
{
   time_t t = time( 0 );
   tm time  = *localtime( &t );
   return ATime( time.tm_hour, time.tm_min, time.tm_sec );
}

// ATime functions
bool ATime::isBetween(const ATime& a, const ATime& b) const
{
   return *this >= a && *this <= b;
}

unsigned long ATime::getHour() const
{
  // seconds/(seconds/hour) -> hours.
  return ((unsigned long)(ticks / 3600));
}

unsigned long ATime::getMinutes() const
{
  // seconds (past hour)/(seconds/minute) -> minutes.
  return ((unsigned long)((ticks % 3600 ) / 60));
}


unsigned long ATime::getSeconds() const
{
  // seconds (past minute)
  return ((unsigned long)(ticks % 60));
}

char* ATime::formatTime() const
{
   static char   timeBuf[9];

   if(nullTime==true)
   {
      sprintf(timeBuf,"NULL");
   }else
   {
      sprintf(timeBuf,"%02d:%02d:%02d",getHour(),getMinutes(),getSeconds());
   }
   return timeBuf;
}

bool ATime::isValid() const
{
   if(nullTime==true) return false;
   return true;
}

// protected
// Operations

// private
// Operations
void ATime::initialize( long aNumberOfTicks)
{
   const long secondsPerDay = 24*60*60L;
   if ( aNumberOfTicks < 0 )
   {
      // if aNumberOfTicks > -24 hour, remember only the seconds before 00:00:00 but in the normal format
      ticks = secondsPerDay - (-aNumberOfTicks % secondsPerDay);
   } else
   {
      // if aNumberOfTicks > 24 hour, remember only the seconds after 00:00:00
      ticks = aNumberOfTicks % secondsPerDay;
   }
}

// End of class definition of ATime

// Friend output operator
ostream& ALIBIMPEXP operator<<(ostream& os,const ATime& aATime)
{
   return os << aATime.formatTime();
}

ATime ALIBIMPEXP operator+(const ATime& aATime, long aSeconds)
{
   return ATime(  aATime.ticks + aSeconds);
}

ATime ALIBIMPEXP operator+(const ATime& aATime, const ATime& bATime)
{
   return ATime(  aATime.ticks + bATime.ticks);
}

ATime ALIBIMPEXP operator+(long aSeconds, const ATime& aATime)
{
   return ATime(  aATime.ticks + aSeconds);
}

ATime ALIBIMPEXP operator-(const ATime& aATime, long aSeconds)
{
   return ATime(  aATime.ticks - aSeconds);
}

ATime ALIBIMPEXP operator-(const ATime& aATime, const ATime& bATime)
{
   return ATime(  aATime.ticks - bATime.ticks);
}


ATime ALIBIMPEXP operator-(long aSeconds, const ATime& aATime)
{
   return ATime(  aATime.ticks - aSeconds);
}

