#ifndef _ATime_hpp_
#define _ATime_hpp_

// Copyright (c) 1997-1999 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to JKraaijeveld@Askesis.nl

// Author : Joost Kraaijeveld
// Created: 26-09-97 14:12:24
// Purpose:

// Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers
#include <iostream>
#include <time.h>
#include <stdio.h>

// Operating system toolkit headers

// STL headers

// Other class headers
#include "ALibObject.hpp"

// Forward declarations

class ALIBIMPEXP ATime : virtual public ALibObject
{
   public:
      // Constructors
      ATime();                                  // current time
      ATime(long aSeconds);                     // Seconds since midnight, can be negative
      ATime(const char* aFormatedTimeString);   // Must be in the format hh:mm:ss
      ATime(unsigned long aHour,
            unsigned long aMinutes,
            unsigned long aSeconds = 0);
      ATime(const ATime& aATime);

      // Destructor

      // Operators
      ATime& operator=(const ATime& aATime);
      bool operator==(const ATime& aATime) const;
      bool operator<(const ATime& aATime) const                   { return ticks < aATime.ticks; }
      bool operator<=(const ATime& aATime) const                  { return ticks <= aATime.ticks; }
      bool operator>(const ATime& aATime) const                   { return ticks > aATime.ticks; }
      bool operator>=(const ATime& aATime) const                  { return ticks >= aATime.ticks; }
      bool operator!=(const ATime& aATime) const                  { return ticks != aATime.ticks; }

      // These functions return a ATime. These returned ATime's DO NOT represent the time after 00:00:00
      // but are a conveniant way of notating time. Actually the whole idea of arithmatic with time is strange
      // TODO: see if the ATime and ADate classes  must have two accompanying classes so we that get a time class
      // that represents an absolute time, a time class that represent duration, and the same for dates
      friend ATime operator+(const ATime& aATime, long aSeconds);
      friend ATime operator+(const ATime& aATime, const ATime& bATime);
      friend ATime operator+(long aSeconds, const ATime& aATime);
      friend ATime operator-(const ATime& aATime, long aSeconds);
      friend ATime operator-(const ATime& aATime, const ATime& bATime);
      friend ATime operator-(long aSeconds, const ATime& aATime);

      ATime& operator+=(long aSeconds);
      ATime& operator+=(const ATime& aATime);
      ATime& operator-=(long aSeconds);
      ATime& operator-=(const ATime& aATime);

      // Accessors
      unsigned long                             getTicks() const {return ticks;}

      // Mutators
      void                                      setNullTime(bool aNullTime = true){nullTime = aNullTime;}

      // Other operations
      // ITime compatible functions
//      IString                                   asString() const;
//      IString                                   asString( const char* fmt) const;
      static ATime                              now();
      // ATime functions
      bool                                      isBetween(  const ATime& a,
                                                            const ATime& b) const;
      unsigned long                             getHour() const;           // hour in local time
      unsigned long                             getMinutes() const;        // minute in local time
      unsigned long                             getSeconds() const;        // second in local time or GMT
      bool                                      isValid() const;
      char*                                     formatTime() const;

      // Attributes

   protected:
      // Attributes

      // Operations

   private:
      // Friends
      friend ostream& operator<<(ostream& os,const ATime& aATime);

      // Operations
      void                                      initialize( long aNumberOfTicks);

      // Attributes
      unsigned long                             ticks;
      bool                                      nullTime;
};
#endif

