#include "ADate.hpp"

// IBM Bug in strcpy() fix
#ifdef strcpy
#undef strcpy
#define strcpy(a,b) strncpy(a,b,strlen(b)+1)
#endif

#define ABBR_LENGTH 3

static const char* dayname[] = { "Sunday",
                                 "Monday",
                                 "Tuesday",
                                 "Wednesday",
                                 "Thursday",
                                 "Friday",
                                 "Saturday"} ;

static const char* mname[] = {   "January",
                                 "February",
                                 "March",
                                 "April",
                                 "May",
                                 "June",
                                 "July",
                                 "August",
                                 "September",
                                 "October",
                                 "November",
                                 "December"};

static int gauDays[] = {31,
                        28,
                        31,
                        30,
                        31,
                        30,
                        31,
                        31,
                        30,
                        31,
                        30,
                        31 };

// public
// Constructors
ADate::ADate()
   :  nullDate(false)
{
  displayFormat=wxMDY;
  displayOptions='\0';
//  month = day = year = day_of_week = 0;
//  julian = 0;
   set();
}

ADate::ADate(const long aJulianDate)
   :  julian(aJulianDate),
      nullDate(false)
{
  displayFormat=wxMDY;
  displayOptions='\0';
  julian_to_mdy ();
}

ADate::ADate(  const int aDay,
               const int aMonth,
               const int aYear)
   :  day(aDay),
      month(aMonth),
      year(aYear),
      nullDate(false)
{
  displayFormat=wxMDY;
  displayOptions='\0';
  mdy_to_julian ();
}

ADate::ADate( const char* aFormatedDateString)
   :  nullDate(false)
{
  displayFormat=wxMDY;
  displayOptions='\0';

   if(strlen(aFormatedDateString)==0)
   {
      month = day = year = day_of_week = 0;
      julian = 0;
      nullDate = true;
   }
   else
   {
      //  0123456789
      // "1990-01-01"
      //  yyyy-mm-dd
      set(  atoi(aFormatedDateString+8),     // dd
            atoi(aFormatedDateString+5),     // mm
            atoi(aFormatedDateString+0));    // yyyy
   }
}


ADate::ADate(const ADate& aADate)
{
   displayFormat  = aADate.displayFormat;
   displayOptions = aADate.displayOptions;
   month          = aADate.month;
   day            = aADate.day;
   year           = aADate.year;
   nullDate       = aADate.nullDate;
   mdy_to_julian ();
}

// Destructor

// Operators
ADate& ADate::operator=(const ADate& aADate)
{
   displayFormat  = aADate.displayFormat;
   displayOptions = aADate.displayOptions;
   month          = aADate.month;
   day            = aADate.day;
   year           = aADate.year;
   nullDate       = aADate.nullDate;
   mdy_to_julian ();
   return *this;
}

ADate::operator string( void )
{
   return formatDate();
}

ADate ADate::operator+(const long i)
{
   ADate dp(julian + i);
   return dp;
}

ADate ADate::operator+(const int i)
{
   ADate dp(julian + (long)i);
   return dp;
}

ADate ADate::operator-(const long i)
{
   ADate dp(julian - i);
   return dp;
}

ADate ADate::operator-(const int i)
{
   ADate dp(julian - (long)i);
   return dp;
}

long ADate::operator-(const ADate &dt)
{
   return ( julian - dt.julian );
}

ADate& ADate::operator+=(const long i)
{
   julian += i;
   julian_to_mdy();
   return *this;
}

ADate& ADate::operator-=(const long i)
{
   julian -= i;
   julian_to_mdy();
   return *this;
}

ADate& ADate::operator++()
{
   julian++;
   julian_to_mdy();
   return *this;
}

ADate& ADate::operator++(int)
{
   julian++;
   julian_to_mdy();
   return *this;
}

ADate& ADate::operator--()
{
   julian--;
   julian_to_mdy();
   return *this;
}

ADate& ADate::operator--(int)
{
   julian--;
   julian_to_mdy();
   return *this;
}

bool ADate::operator<(const ADate& aADate) const
{
   return ( julian < aADate.julian );
}

bool ADate::operator<=(const ADate& aADate) const
{
   return ( (julian == aADate.julian) || (julian < aADate.julian) );
}

bool ADate::operator>(const ADate& aADate) const
{
   return ( julian > aADate.julian );
}

bool ADate::operator>=(const ADate& aADate) const
{
   return ( (julian == aADate.julian) || (julian > aADate.julian) );
}

bool ADate::operator==(const ADate& aADate) const
{
   return ( (julian == aADate.julian) && (nullDate == aADate.nullDate) );
}

bool ADate::operator!=(const ADate& aADate) const
{
   return ( julian != aADate.julian );
}

// Accessors
long ADate::getJulianDate( void ) const
{
   return julian;
}

int ADate::getYear() const
{
    return year;
}

int ADate::getMonth() const
{
    return month;
}

int ADate::getDay() const
{
    return day;
}

// Mutators

// Other operations
// IDate function

//IString ADate::asString() const
//{
//   return IString(formatDate().c_str());
//}

//IString ADate::asString(YearFormat /*yearFmt*/) const
//{
//   return IString(formatDate().c_str());
//}

//IString ADate::asString(const char* /*fmt*/) const
//{
//   return IString(formatDate().c_str());
//}

string ADate::formatDate(const int type) const
{
   int actualType = type;
   if (actualType == -1)
   {
      actualType = displayFormat;
   }

   char buf[40];
   memset( buf, '\0', sizeof(buf));

   switch (actualType)
   {
      case wxDAY:
      {
         if ( (day_of_week < 1) || (day_of_week > 7) )
         {
            strcpy(buf,"invalid day");
         }else
         {
            strncpy( buf, dayname[day_of_week-1],(displayOptions & wxDATE_ABBR) ? ABBR_LENGTH : 9);
         }
         return string(buf);
//         break;
      }
      case wxMONTH:
      {
         if ( (month < 1) || (month > 12) )
         {
            strcpy(buf,"invalid month");
         }else
         {
            strncpy( buf, mname[month-1],(displayOptions & wxDATE_ABBR) ? ABBR_LENGTH : 9);
         }
         return string(buf);
//         break;
      }
      case wxFULL:
      {
         if ( (month < 1) || (month > 12) || (day_of_week < 0) || (day_of_week > 7) )
         {
            strcpy(buf,"invalid date");
            return string(buf);
//            break;
         }
         strncpy( buf, dayname[day_of_week-1],(displayOptions & wxDATE_ABBR) ? ABBR_LENGTH : 9);
         strcat( buf, ", ");
         strncat( buf, mname[month-1],(displayOptions & wxDATE_ABBR) ? ABBR_LENGTH : 9);
         strcat( buf, " ");
         sprintf( buf+strlen(buf), "%d, %d", day, abs(year) );
         if (year < 0)
         {
            strcat(buf," B.C.");
         }
         return string(buf);
//         break;
      }
      case wxEUROPEAN:
      {
         if ( (month < 1) || (month > 12) || (day_of_week < 0) || (day_of_week > 7) )
         {
            strcpy(buf,"invalid date");
            return string(buf);
//            break;
         }
         sprintf(buf,"%d ",day);
         strncat(buf, mname[month-1], (displayOptions & wxDATE_ABBR) ? ABBR_LENGTH : 9);
         sprintf( buf+strlen(buf), " %d", abs(year) );
         if (year < 0)
         {
            strcat(buf," B.C.");
         }
         return string(buf);
//         break;
      }
      case wxMDY:
      default:
      {
         if (day==0 || month==0 || year==0)
         {
            strcpy(buf,"invalid date");
         }else
         {
            sprintf( buf+strlen(buf), "%02d-%02d-%04d",day, month,  (displayOptions & wxNO_CENTURY) && (abs(year) > 1899) ? (abs(year) - (abs(year) / 100 * 100)) : (abs(year)));
         }
         return string(buf);
//         break;
      }
   }
//  return string("");
}

void ADate::setFormat( const int format )
{
   displayFormat = format;
}

int ADate::setOption( const int option, const bool action )
{
   switch ( option )
   {
      case wxNO_CENTURY:
      {
         if(action)
         {
            displayOptions |= wxNO_CENTURY;
         }else
         {
           displayOptions &= (~wxNO_CENTURY);
         }
         return 1;
//         break;
      }
      case wxDATE_ABBR:
      {
         if(action)
         {
            displayOptions |= wxDATE_ABBR;
         }else
         {
            displayOptions &= (~wxDATE_ABBR);
         }
         return 1;
//         break;
      }
      default:
      {
         return 0;
//         break;
      }
   }
//   return 0;
}

int ADate::getDayOfYear( void ) const
{
   ADate temp( 1, 1, year );
   return (int) (julian - temp.julian + 1);
}


bool ADate::isLeapYear( void ) const
{
   return  ( (year >= 1582) ? (year % 4 == 0  &&  year % 100 != 0  ||  year % 400 == 0 ): (year % 4 == 0) );
}

ADate& ADate::set()
{
   time_t now  = time(NULL);
   struct tm *localTime = localtime(&now);
   month = localTime->tm_mon + 1;
   day   = localTime->tm_mday;
   year  = localTime->tm_year + 1900;
   mdy_to_julian ();
   return *this;
}

ADate& ADate::set(int  aDay,
                  int  aMonth,
                  int  aYear)
{
   month = aMonth;
   day   = aDay < getDaysInMonth() ? aDay : getDaysInMonth();
   year  = aYear < 0 ? 9999 : aYear;
   year  = aYear > 9999 ? 0 : aYear;
   mdy_to_julian();
   return *this;
}

ADate& ADate::set(long j)
{
  julian = j;
  julian_to_mdy();
  return *this;
}


int ADate::getDaysInMonth()
{
   int rv = gauDays[month-1];
   if(month==2)
   {
      if(isLeapYear())
      {
         rv++;
      }
   }
   return rv;
//   return gauDays[month-1] + (month==2 && isLeapYear());
}

int ADate::getFirstDayOfMonth() const
{
    return ADate(month, 1, year).getDayOfWeek();
}

int ADate::getDayOfWeek() const
{
    return day_of_week;
}

ADate& ADate::addWeeks(int nCount)
{
    set(julian + (long)nCount*7);
    return *this;
}

ADate& ADate::addMonths(int nCount)
{
   month += nCount;

   if (month < 1)
   {
      month = 12;
      year--;
   }

   if (month > 12)
   {
      month = 1;
      year++;
   }
   mdy_to_julian();
   return *this;
}

ADate& ADate::addYears(int nCount)
{
    year += nCount;
    mdy_to_julian();
    return *this;
}

int ADate::getWeekOfMonth()
{
  // Abs day includes the days from previous month that fills up
  // the begin. of the week.
  int nAbsDay = day + getFirstDayOfMonth()-1;
  return (nAbsDay-getDayOfWeek())/7 + 1;
}

int ADate::getWeekOfYear()
{
    ADate   doTemp(1, 1, year);
    return (int)(((julian - doTemp.julian+1)/7) + 1);
}

ADate ADate::getMonthStart()
{
    return(ADate(month, 1, year));
}

ADate ADate::getMonthEnd()
{
    return(ADate(month+1, 1, year)-1);
}

ADate ADate::getYearStart()
{
    return(ADate(1, 1, year));
}

ADate ADate::getYearEnd()
{
    return(ADate(1, 1, year+1)-1);
}

string ADate::getMonthName()
{
    return(formatDate(wxMONTH));
}

string ADate::getDayOfWeekName()
{
    return(formatDate(wxDAY));
}

bool ADate::isBetween(const ADate& first, const ADate& second) const
{
  return (julian >= first.julian && julian <= second.julian);
}

ADate ADate::previous(const int dayOfWeek) const
{
   int this_day_Of_Week, desired_day_Of_Week;
   long j;

//      Set the desired and current day of week to start at 0 (Monday)
//      and end at 6 (Sunday).

   desired_day_Of_Week = dayOfWeek - 1; // These functions return a value
   this_day_Of_Week    = getDayOfWeek() - 1;            // from 1-7.  Subtract 1 for 0-6.
   j = julian;

//      Have to determine how many days difference from current day back to
//      desired, if any.  Special calculation under the 'if' statement to
//      effect the wraparound counting from Monday (0) back to Sunday (6).

   if (desired_day_Of_Week > this_day_Of_Week)
   {
      this_day_Of_Week += 7 - desired_day_Of_Week;
   }else
   {
      this_day_Of_Week -= desired_day_Of_Week;
   }
   j -= this_day_Of_Week; // Adjust j to set it at the desired day of week.
   return ADate(j);
}

bool ADate::isValid() const
{
   if(nullDate == true) return false;
   return true;
}

// protected
// Operations

// private
// Operations
void ADate::julian_to_wday (void)
{
   day_of_week = (int) ((julian + 2) % 7 + 1);
}

void ADate::julian_to_mdy ()
{
   long a,b,c,d,e,z,alpha;
   z = julian+1;
   // dealing with Gregorian calendar reform
   if (z < 2299161L)
   {
      a = z;
   }else
   {
      alpha = (long) ((z-1867216.25) / 36524.25);
      a = z + 1 + alpha - alpha/4;
   }
   b = ( a > 1721423 ? a + 1524 : a + 1158 );
   c = (long) ((b - 122.1) / 365.25);
   d = (long) (365.25 * c);
   e = (long) ((b - d) / 30.6001);
   day = (int)(b - d - (long)(30.6001 * e));
   month = (int)((e < 13.5) ? e - 1 : e - 13);
   year = (int)((month > 2.5 ) ? (c - 4716) : c - 4715);
   julian_to_wday ();
}

void ADate::mdy_to_julian (void)
{
   int a,b=0;
   int work_month=month, work_day=day, work_year=year;
   // correct for negative year
   if (work_year < 0)
   {
      work_year++;
   }
   if (work_month <= 2)
   {
      work_year--; work_month +=12;
   }
   // deal with Gregorian calendar
   if (work_year*10000. + work_month*100. + work_day >= 15821015.)
   {
      a = (int)(work_year/100.);
      b = 2 - a + a/4;
   }
   julian = (long) (365.25*work_year) + (long) (30.6001 * (work_month+1))  +  work_day + 1720994L + b;
   julian_to_wday ();
}

// End of class definition of ADate

// Friend output operator
ostream& /*ALIBIMPEXP*/ operator<<(ostream& os,const ADate& aADate)
{
   // "1990-01-01"
   return os << aADate.formatDate();
}
